#! /usr/local/bin/perl
#############################################################################
#  
#   File:          ReadBoard.pl
#
#   Author:        Jungmin Kim
#
#   Created:       03/05/95
#   Modified:      
#   Version: 
#
#   Input:         path name, file name, user information, subdirectory
#                  $path, $file, $user, $subdir
#   Output:        HTML text
#
#   Description:   Print out headers of messages for a particular conference
#                   (given by the path and specified files). 
#                  Function PrintHeader is for printing headers. 
#                  Function PrintFile is for printing messages.
#                  Function dotorder sorts file names in hierachical numbering
#                   order.
#               
#   Usage:         "http://..../WeBBoard/read:path:filename:user"
#   Usage:         "http://..../WeBBoard/read:path:filename::user" ??
#
#                  'user' should be changed to "username:email_address".
#
# BUGS   - is not yet handling username:email_address and printing user's
#          email addresses next to their names
#        - some strange bug where it thinks $user read in from the index
#          file is actually $subdir (FIXED?)
#
#############################################################################

require "source/CheckPasswd.pl";
require "source/PrintHeader.pl";
require "source/PrintHeader2.pl";
require "source/PrintFile.pl";

sub ReadBoard{

    local($path, $file, $user, $subdir) = @_;
    local($prev, @entry, %array, $d, $t, $n, $pathname, $message);
    local($oldm, $oldn, $oldd, $oldt);
    local(%messages, %titles, %dates, %names);

    print "Content-type\: text/html\n\n";
    print "\n";
    print "<TITLE> READ MESSAGES </TITLE>\n";
    print "<H2> Read messages for the forum </H2>\n";
    $pathname = $conference{$path};
    print "<H1>$pathname</H1>\n";
	
    print "<HR>\n";
   # explain icon for adding new message
    if($user ne "guest") {
	print "<IMG SRC=\"$gif/pencil.gif\" ALIGN=\"left\" ALT=\"\">";
        # Ideally this would be in italics but italics look terrible on
        #  Netscape for the Mac
	print "Click on this icon, which appears next to each message, <BR>\n";
        print "to respond to that message or post a new message. \n";
        print "<P>\n";
    }

    print "<IMG SRC=\"$gif/pointer.xbm\" ALIGN=\"left\" ALT=\"\">";
    print "<A HREF=\"$action/$subdir\:select\:$path\:\:$user\">\n";
    print "<B>RETURN to the index of forums</B></A>\n";
    print "<P>\n";
    print "<IMG SRC=\"$gif/pointer.xbm\" ALIGN=\"left\" ALT=\"\">";
    print "<A HREF=\"#subject\">\n";
    print "<B>JUMP to the list of messages ordered by subject</B></A>\n";
    print "<HR>\n";
    print "<HR>\n";

    #### array for file names to be printed
    %array = ();


    #### print out all headers of messages in directory $path in reverse
    #### time order
    print "<B>List of messages ordered by time of posting</B>\n";
    &PrintHeader($path, $file, $user, $subdir, *array);
    print "<HR>\n";
    print "<HR>\n";

    #### print out all headers of messages in directory $path in numerical
    #### order
    print "<A NAME = subject>\n";
    print "<B>List of messages ordered by subject of posting</B></A>\n";
    &PrintHeader2($path, $file, $user, $subdir);
    print "<HR>\n";
    print "<HR>\n";

    $initial = "";
    #### print messages coresponding to returned file names
    foreach $next (sort dotorder keys %array) {
	$_ = $next;

	# indentation for hierachical view
	print "<DL>\n";
	while ( /^[0-9]*\.(.*)/ ) {  # number of dots is number of tabs
	    $_ = $1;
	    print "<DL>\n";
	}

	print "<A NAME=\"$next\">\n";

        # separator for each field is eot(\005) *************** modified
	#### split the passed string associated to the file name
        #### into date ($d), name ($n) and title ($t)
	($d, $n, $t) = split(/\005/, $array{$next});
        
        #### Get email address to go with user name
        $e = GetAddress($subdir, $n);

	#### if the user name is 'guest', do not print out submit button

	$pathname = "$webboardhome\/$subdir\/$path";
	$message = &PrintFile($pathname, "$next.html");

	$oldm = $message;
	$oldt = $t;
        $oldn = $n;
	$oldd = $d;

        ### This could be where things screw up - it replaces 
        ### special characters such as : and +
	$d = &Replace($d);
	$n = &Replace($n);
	$t = &Replace($t);
	$message = &Replace($message);

	$initial = $1 if $next =~ /^(.+)\.([^\.]*)/;
	$initial = $next if $next =~ /^[^\.]+$/;

	$messages{$next} = $message;
	$titles{$next} = $t;
	$dates{$next} = $d;
	$names{$next} = $n;

	print "<DT>\n";
	
	# icon for adding new message
        if($user ne "guest") {
	    print "<A HREF=\"$action/$subdir\:post\:$path\:$next\:$user";
	    print "\?hdate=$d\&hname=$n\&htitle=$t\&hmessage=$message";
	    print "\&initial=$initial";
	    print "\&htitle2=$titles{$initial}";
	    print "\&hmessage2=$messages{$initial}";
	    print "\&hdate2=$dates{$initial}";
	    print "\&hname2=$names{$initial}\">\n";
	    print "<IMG SRC=\"$gif/pencil.gif\" ALIGN=\"bottom\" ALT=\"\">";
	    print "</A>\n";
	}

	print "<b>$next\:</b> ";

	# Changed back so it doesn't print title on new line
	print "From <B>$oldn <I>(<A HREF=\"mailto:$e\">$e</A>)</I></B>\n";
	print "on $oldd\n";
	print "regarding <B>$oldt</B><BR>\n";
        # **************************************************************
	
	print "<P>\n";

	#### print out message
	print $oldm;
	print "<P><HR>\n";

	#### close indentation
	$_ = $next;
	while ( /^[0-9]*\.(.*)/ ) { # number of dots is number of tabs
	    $_ = $1;
	    print "</DL>\n";
	}
	print "</DL>\n";
    }
    print "<HR>\n";
    Address();
}
1;

#############################################################################
#
#   dotorder
#
#   INPUT:     $a, $b (system arguments passed by function sort(). )
#   OUTPUT:    greater than, less than, or equal
#
#   FUNCTION:  Compare $a and $b with hierachical numbering order, 
#              and returns -1, 0, 1.
#
#############################################################################

sub dotorder {
    $c = $a;
    $d = $b;

    for(;;){
	$c =~ s/^(\d*)(\.)*//;            # remove prefix of $a
	$e = $1;		          # copy the prefix of $a
	$d =~ s/^(\d*)(\.)*//;            # remove prefix of $b
	$f = $1;		          # copy the prefix of $b
	last if $e != $f;	          # compare prefixes of each numbers
    }
    return $e <=> $f;
}
1;

#############################################################################
#
#   Replace
#
#   INPUT:     string
#   OUTPUT:    exactly same format as 'QUERY_STRING' in 'GET' method
#
#   FUNCTION:  convert special characters to hex numbers
#
#############################################################################

sub Replace {
    local($st) = @_;
    $_ = $st;
    s/\%/%25/g;
    s/\n/%0A/g;
    s/\r/%0D/g;
    s/\t/%09/g;
    s/\!/%21/g;
    s/\"/%22/g;
    s/\#/%23/g;
    s/\$/%24/g;
    s/\&/%26/g;
    s/\'/%27/g;
    s/\(/%28/g;
    s/\)/%29/g;
    s/\+/%2B/g;
    s/\,/%2C/g;
    s/\//%2F/g;
    s/\:/%3A/g;
    s/\;/%3B/g;
    s/\</%3C/g;
    s/\=/%3D/g;
    s/\>/%3E/g;
    s/\?/%3F/g;
    s/\[/%5B/g;
    s/\\/%5C/g;
    s/\]/%5D/g;
    s/\^/%5E/g;
    s/\`/%60/g;
    s/\{/%7B/g;
    s/\|/%7C/g;
    s/\}/%7D/g;
    s/\~/%7E/g;
    s/ /\+/g;
    $_;
}
1;
